
/*              Copyright 1993 Colorado State University                    */
/*                      All Rights Reserved                                 */

/*****************************************************************************
**
**  FILE:      initsite_tg.c
**
**  FUNCTION:  void initsite()
**
**  PURPOSE:   Read in site specific parameters (from sitepar.in)
**
**  AUTHOR:    Susan Chaffee    March 10, 1992
**
**  REWRITE:   Melannie Hartman  9/9/93 - 9/23/93
**
**  HISTORY:
**    8/13/92 (SLC) - Change the way the parameter for minimum soil water
**                    content is used.  No longer a function of wilting point,
**                    now it simply gives the percent of water at each layer.
**    5/08/08 (CAK) - Add checks for end of file, if the end of the file is
**                    reached before all of the variables have been read and
**                    initialized this will result in a fatal error.
**
**  INPUTS:
**    flags          - structure containing debugging flags
**    flags->debug   - flag to set debugging mode, 0 = off, 1 = on
**    flags->verbose - flag to set verbose debugging mode, 0 = off, 1 = on
**    sitename       - data file name containing site parameters
**    sitepar        - site specific parameters structure for soil water model
**    layers         - soil water soil layer structure
**
**  GLOBAL VARIABLES:
**    INPTSTRLEN - maximum length of input file line (120)
**    NTDEPTHS   - maximum number of soil regions (4)
**
**  LOCAL VARIABLES:
**    errmsg[]   - string containing error message
**    fp_in      - pointer to input file
**    imo        - current month (1..12)
**    inptline[] - line read from input file
**    m          - month
**
**  OUTPUTS:
**    Read from file sitename:
**    sitepar->albedo        - fraction of light reflected by snow
**    sitepar->cldcov[]      - average cloud cover for the month (%, 1..100)
**    sitepar->dmp           - damping factor for calculating soil temperature
**                             by layer
**    sitepar->dmpflux       - damping factor for soil water flux (in h2oflux)
**    sitepar->drainlag      - number of days that soil drainage should lag
**                             behind rain/irrigation/melt events (1-5)
**    sitepar->fswcinit      - initial soil water content, fraction of field
**                             capacity (0.0 - 1.0)
**    sitepar->hours_rain    - the duration of the rainfall/snowmelt event
**                             (hours)
**    sitepar->hpotdeep      - hydraulic water potential of deep storage
**                             layer, the more negative the number the dryer
**                             the soil layer (units?)
**    sitepar->jdayEnd       - the Julian day to end the turning off of the
**                             restriction of the CO2 effect on
**                             denitrification
**    sitepar->jdayStart     - the Julian day to start the turning off of the
**                             restriction of the CO2 effect on
**                             denitrification
**    sitepar->ksatdeep      - saturated hydraulic conductivity of deep
**                             storage layer (cm/sec)
**    sitepar->MaxNitAmt     - maximum daily nitrification amount (gN/m^2)
**    sitepar->N2N2Oadj      - multiplier on N2/N2O ratio
**    sitepar->NO3_N2O_slope - slope, controls how fast the percentage of NO3
**                             from mineralization that is lost as N2O changes
**                             as a function of water filled pore space
**    sitepar->NO3_N2O_step  - step between minimum and maximum percentage of
**                             NO3 from mineralization that is lost as N2O
**    sitepar->NO3_N2O_x     - x-inflection, water filled pore space where 1/2
**                             of maximum percentage of NO3 from
**                             mineralization is lost as N2O
**    sitepar->NO3_N2O_y     - y-inflection, percentage of NO3 from
**                             mineralization that is lost as N2O at
**                             x-inflection water filled pore space
**    sitepar->Ncoeff        - minimum water/temperature limitation
**                             coefficient for nitrification
**    sitepar->netmn_to_no3  - fraction of new net mineralization that goes to
**                             NO3 (0.0-1.0)
**    sitepar->reflec        - fraction of light reflected by vegetation
**    sitepar->SnowFlag      - effect of snow on soil surface temperature calc
**                             1 = insulating effect, 0 = no insulating effect
**    sitepar->sublimscale   - multiplier to scale sublimation
**    sitepar->tbotmn        - minimum emperature for bottom soil layer
**                             (degrees C)
**    sitepar->tbotmx        - maximum emperature for bottom soil layer
**                             (degrees C)
**    sitepar->texture       - texture classification for trace gas model
**                             (1 = coarse, 2 = medium, 3 = fine)
**    sitepar->timlag        - days from Jan 1 to coolest temp at bottom of
**                             soil (days)
**    sitepar->usexdrvrs     - 1 = use extra drivers (solrad, rel humid,
**                             windsp) for PET calculation, 0 = use air
**                             temperature to drive PET rates
**    sitepar->watertable[]  - 1 = simulate water table, 0 = no water table
**    sitepar->wfpsdnitadj   - adjustment on inflection point for water filled
**                             pore space effect on denitrification curve
**
**  EXAMPLE INPUT FILE:
**  0        / 1 = Use extra weather drivers (solrad, rhumid, windsp), 
**             0 = don't use (for PET) 
**  1.0      / sublimscale
**  0.18     / reflec - vegetation reflectivity/albedo (frac)
**  0.65     / albedo - snow albedo (frac)
**  0.90     / fswcinit - initial swc, fraction of field capacity
**  0.000001 / dmpflux - in h2oflux routine (0.000001 = original value)
**  4        / hours_rain - duration of each rain event
**  0        / # of days between rainfall event and drainage of soil (-1=computed)
**  1  0     / watertable[month] - 0 = no water table, 1 = water table
**  2  0
**  3  0
**  4  0
**  5  0
**  6  0
**  7  0
**  8  0
**  9  0
**  10 0
**  11 0
**  12 0
**  -200     / hpotdeep - hydraulic water potential of deep storage layer
**             (units?)
**  0.0002   / ksatdeep - saturated hydraulic conductivity of deep storage
**             layer (cm/sec)
**  1  58    / cldcov[month] - cloud cover (%)
**  2  58
**  3  58
**  4  58
**  5  58
**  6  58
**  7  58
**  8  58
**  9  58
**  10 58
**  11 58
**  12 58
**  5.0 16.4 / min and max temperature for bottom soil layer (degrees C)
**  0.003    / damping factor for calculating soil temperature by layer
**  30.0     / timlag, days from Jan 1 to coolest temp at bottom of soil (days)
**  0.03     / min water/temperature limitation coefficient for nitrify
**  0 0      / turn off resp restraint on denit between these days of the year
**  4.0      / maximum proportion of nitrified N lost as N2O @ minimum diffusivity
**  0.2      / minimum proportion of nitrified N lost as N2O @ maximum diffusivity
**  0.4      / maximum daily nitrification amount (gN/m^2)
**  1        / snow effect on soil surface temp: 0 = not insulating, 1 = insulating
**  0.2      / fraction of new net mineralization that goes to NO3 (0.0-1.0)
**  1.0      / adjustment on inflection point for WFPS effect on denit
**  1.0      / N2/N2O ratio adjustment coefficient
**
**  CALLED BY:
**    initsw()
**
**  CALLS:
**    None
**
*****************************************************************************/

#include "soilwater.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

    void initsite(char *sitename, SITEPAR_SPT sitepar, LAYERPAR_SPT layers,
                  FLAG_SPT flags)
    {

      int  imo, m, fcnt;
      char inptline[INPTSTRLEN];
      char errmsg[INPTSTRLEN];
      float fval;
      FILE *fp_in;

      if (flags->debug) {
        printf("Entering function initsite\n");
      }

      if ((fp_in = fopen(sitename, "r")) == NULL) {
        sprintf(errmsg, "Cannot open file %s\n", sitename);
        perror(errmsg);
        exit(1);
      }

      /*****************************************************************************************/
      /* Code from Kendrick to check sitepar.in format 
       */

      flags->verbose = 1;

      while (fgets(inptline, INPTSTRLEN, fp_in) != NULL) {
        fcnt = sscanf(inptline, "%f", &fval);
        if(fcnt != 1) {
          printf("WARNING: unable to parse sitein: %s",inptline);
          continue;
        }

        if(strstr(inptline,"usexdrvrs") != NULL) {
          sscanf(inptline, "%d", &sitepar->usexdrvrs);
          if (flags->verbose) {printf("sitepar; usexdrvrs: %1d\n", sitepar->usexdrvrs);}
          if (sitepar->usexdrvrs < 0 || sitepar->usexdrvrs > 1) {
            printf("ERROR:  Invalid usexdrvrs in sitepar.in: %1d\n", sitepar->usexdrvrs);
          }
        }

        else if(strstr(inptline,"sublimscale") != NULL) {
          sitepar->sublimscale = fval;
          if (flags->verbose) {
            printf("sitepar; sublimation scalar sublimscale: %f\n", sitepar->sublimscale);
          }
        }

        else if(strstr(inptline,"reflec") != NULL) {
          sitepar->reflec = fval;
          if (flags->verbose) {
            printf("sitepar; vegetation reflectivity, reflec: %f\n", sitepar->reflec);
          }
        }

        else if(strstr(inptline,"albedo") != NULL) {
          sitepar->albedo = fval;
          if (flags->verbose) {
            printf("sitepar; snow albedo, albedo: %f\n", sitepar->albedo);
          }
        }
 
        else if(strstr(inptline,"fswcinit") != NULL) {
          sitepar->fswcinit = fval;
          if (flags->verbose) {
            printf("sitepar; initial soil water content: %f\n", sitepar->fswcinit);
          }
        }

        else if(strstr(inptline,"dmpflux") != NULL) {
          sitepar->dmpflux = fval;
          if (flags->verbose) {
            printf("sitepar; soil water flux damping factor, dmpflux: %f\n", sitepar->dmpflux);
          }
        }

        else if(strstr(inptline,"hours_rain") != NULL) {
          sitepar->hours_rain = fval;
          if (flags->verbose) {
            printf("sitepar; rainfall duration, hours_rain: %f\n", sitepar->hours_rain);
          }
        }
 
        /* Allow user to set number of days between rainfall event and */
        /* drainage of soil profile.  If a value of -1 is entered set the */
        /* number of days to drainage based in the soil texture.  Constrain */
        /* the number of days to drainage to be <=5 to prevent numerical */
        /* instabilities in the h2oflux subroutine.  cak - 02/13/04 */

        else if(strstr(inptline,"drainlag") != NULL) {
          sitepar->drainlag = fval;
          if (sitepar->drainlag < 0) {
            sitepar->drainlag = sitepar->texture - 1;
          }
          if (sitepar->drainlag > 5) {
            printf("sitepar; lag period for drainage too long, setting to max value\n");
            sitepar->drainlag = 5;
          }
          if (flags->verbose) {
            printf("sitepar; drainlag: %d\n", sitepar->drainlag);
          }
        }

        else if(strstr(inptline,"watertable[1]") != NULL) {
          imo = 1;
          sscanf(inptline, "%d %d", &m, &sitepar->watertable[imo]);
          if (flags->verbose) {
            printf("sitepar; watertable[%1d]: %d\n", imo, sitepar->watertable[imo]);
          }
          for(imo=2; imo<=12; imo++) 
          {
            if (fgets(inptline, INPTSTRLEN, fp_in) != NULL) 
            {
              if(strstr(inptline,"watertable") != NULL) 
              {
                  sscanf(inptline, "%d %d", &m, &sitepar->watertable[imo]);
                  if (flags->verbose) {
                    printf("sitepar; watertable[%1d]: %d\n", imo, sitepar->watertable[imo]);
                  }
              }
              else
              {
                printf("ERROR:  Missing simulate water table flag for month %1d\n", imo);
                printf("in sitepar.in file.  Ending simulation!\n");
                exit(1);
              }
            }
            else 
            {
              printf("ERROR:  Unexpected end of file in sitepar.in.\n");
              printf("Ending simulation!\n");
              exit(1);
            }
          }
        }

        else if(strstr(inptline,"hpotdeep") != NULL) {
          sitepar->hpotdeep = fval;
          if (flags->verbose) {
            printf("sitepar; deep storage hydraulic potential, hpotdeep: %f\n", sitepar->hpotdeep);
          }
        }

        else if(strstr(inptline,"ksatdeep") != NULL) {
          sitepar->ksatdeep = fval;
          if (flags->verbose) {
            printf("sitepar; deep storage saturated hydraulic conductivity, ksatdeep: %f\n", sitepar->ksatdeep);
          }
        }

        else if(strstr(inptline,"cldcov[1]") != NULL) {
          imo = 1;
          sscanf(inptline, "%d %f", &m, &sitepar->cldcov[imo]);
          if (flags->verbose) {
            printf("sitepar; cldcov[%1d]: %f\n", imo, sitepar->cldcov[imo]);
          }
          for(imo=2; imo<=12; imo++) 
          {
            if (fgets(inptline, INPTSTRLEN, fp_in) != NULL) 
            {
              if(strstr(inptline,"cldcov") != NULL) 
              {
                  sscanf(inptline, "%d %f", &m, &sitepar->cldcov[imo]);
                  if (flags->verbose) {
                    printf("sitepar; cldcov[%1d]: %f\n", imo, sitepar->cldcov[imo]);
                  }
              }
              else
              {
                printf("ERROR:  Missing average cloud cover value for month %1d\n", imo);
                printf("in sitepar.in file.  Ending simulation!\n");
                exit(1);
              }
            }
            else 
            {
              printf("ERROR:  Unexpected end of file in sitepar.in.\n");
              printf("Ending simulation!\n");
              exit(1);
            }
          }
        }

        /* The texture parameter is being set in the initlyrs subroutine  CAK - 05/31/01 */
        /* The texture parameter has been replaced by the minimum and */
        /* maximum temperature values for the bottom soil layer */
        else if(strstr(inptline,"tbotmn") != NULL) {
          sscanf(inptline, "%f %f", &sitepar->tbotmn, &sitepar->tbotmx);
          if (flags->verbose) {
            printf("sitepar; min and max temperature for bottom soil layer, tbotmn tbotmx: %f %f\n", sitepar->tbotmn, sitepar->tbotmx);
          }
          if (sitepar->tbotmn > sitepar->tbotmx) {
            printf("ERROR:  base layer minimum temperature is greater than maximum, tbotmn>tbotmx, in sitepar.in\n");
          }
        }

      /* Added dmp parameter to the sitepar.in file, cak - 12/16/02 */
        else if(strstr(inptline,"dmp") != NULL) {
          sitepar->dmp = fval;
          if (flags->verbose) {
            printf("sitepar; soil temperature damping factor, dmp: %f\n", sitepar->dmp);
          }
        }

      /* Added timlag parameter to the sitepar.in file, cak - 04/24/03 */
        else if(strstr(inptline,"timlag") != NULL) {
          sitepar->timlag = fval;
          if (flags->verbose) {
            printf("sitepar; date of coolest temp at soil bottom, timlag: %f\n", sitepar->timlag);
          }
        }

      /* Added Ncoeff parameter to the sitepar.in file, cak - 04/08/03 */
        else if(strstr(inptline,"Ncoeff") != NULL) {
          if(fval > 1.0f) {fval = 1.0f;} else if (fval < -1.0f) {fval = -1.0f;}
          sitepar->Ncoeff = fval;
          if (flags->verbose) {
            printf("sitepar; coefficient for nitrification, Ncoeff: %f\n", sitepar->Ncoeff);
          }
        }

        else if(strstr(inptline,"jdayStart") != NULL) {
          sscanf(inptline, "%d %d", &sitepar->jdayStart, &sitepar->jdayEnd);
          if (flags->verbose) {
            printf("sitepar; denitrification respiration restraint dates, jdayStart/jdayEnd: %d %d\n",
                   sitepar->jdayStart, sitepar->jdayEnd);
          }
        }


        /* Added 4 coefficients to parameterize the arctangent function that */
        /* used to control the amount of NO3 from mineralization is lost as */
        /* N2O, cak - 08/28/2014 */

        else if(strstr(inptline,"NO3_N2O_x") != NULL) {
          sitepar->NO3_N2O_x = fval;
          if (flags->verbose) {
            printf("sitepar; WFPS where 1/2 of max NO3 is lost as N2O: ");
            printf("sitepar->NO3_N2O_x: %f\n", sitepar->NO3_N2O_x);
          }
        }

        else if(strstr(inptline,"NO3_N2O_y") != NULL) {
          sitepar->NO3_N2O_y = fval;
          if (flags->verbose) {
            printf("sitepar; Percent NO3 lost as N2O at x-inflection: ");
            printf("sitepar->NO3_N2O_y: %f\n", sitepar->NO3_N2O_y);
          }
        }

        else if(strstr(inptline,"NO3_N2O_step") != NULL) {
          sitepar->NO3_N2O_step = fval;
          if (flags->verbose) {
            printf("sitepar; Step between min and max NO3 lost as N2O: ");
            printf("sitepar->NO3_N2O_step: %f\n", sitepar->NO3_N2O_step);
          }
        }

        else if(strstr(inptline,"NO3_N2O_slope") != NULL) {
          sitepar->NO3_N2O_slope = fval;
          if (flags->verbose) {
            printf("sitepar; Slope of NO3/N2O WFPS funtion: ");
            printf("sitepar->NO3_N2O_slope: %f\n", sitepar->NO3_N2O_slope);
          }
        }

        /* coefficient to control the maximum daily nitrification amount - cak 09/21/2011 */
        else if(strstr(inptline,"MaxNitAmt") != NULL) {
          sitepar->MaxNitAmt = fval;
          if (flags->verbose) {
            printf("sitepar; maximum daily nitrification, MaxNitAmt: %f\n", sitepar->MaxNitAmt);
          }
        }

        /* flag to turn off the insulating effect of snow on soil surface temperature - cak 10/07/2011 */
        else if(strstr(inptline,"SnowFlag") != NULL) {
          sscanf(inptline, "%d", &sitepar->SnowFlag);
          if (flags->verbose) {
            printf("sitepar; Snow insulating flag, SnowFlag: %d\n", sitepar->SnowFlag);
          }
        }

        /* coefficient to control the fraction of new net mineralization goesing to NO3 - cak 01/09/2012 */
        else if(strstr(inptline,"netmn_to_no3") != NULL) {
          sitepar->netmn_to_no3 = fval;
          if (flags->verbose) {
            printf("sitepar; Fraction of net mineralization to NO3, netmn_to_no3: %f\n", sitepar->netmn_to_no3);
          }
        }

        /* inflection point for the water filled pore space effect on denitrification curve - cak 01/06/2013 */
        else if(strstr(inptline,"wfpsdnitadj") != NULL) {
          sitepar->wfpsdnitadj = fval;
          if (flags->verbose) {
            printf("sitepar; WFPS inflection point adjustment, wfpsdnitadj: %f\n", sitepar->wfpsdnitadj);
          }
        }

        /* Added N2/N2O ratio adjustment coefficient - cak 01/06/2013 */
        else if(strstr(inptline,"n2n2oadj") != NULL) {
          sitepar->n2n2oadj = fval;
          if (flags->verbose) {
            printf("sitepar; N2/N2O ratio adjustment, n2n2oadj: %f\n", sitepar->n2n2oadj);
          }
        }
  
        /* unable to find a variable name */
        else {
          printf("ERROR: Unknown sitepar record: %s", inptline);
          exit(1);
        }
      }

      /*****************************************************************************************/

      fclose(fp_in);

      if (flags->debug) {
        printf("Exiting function initsite\n");
      }

      return;
    }
