
/*              Copyright 1993 Colorado State University                    */
/*                      All Rights Reserved                                 */

/*****************************************************************************
** 
**  FILE:      trwtavg.c
** 
**  FUNCTION:  float trwtavg()
** 
**  PURPOSE:   Compute weighted average of soil water potential to be
**             used for transpiration calculations.
** 
**  AUTHOR:    Susan Chaffee  4/30/92
**
**  REWRITE:   Melannie Hartman  9/20/93 - 9/20/93
**
**  HISTORY:
**    11/09/95 (MDH) Added PILPS modifications.
**    05/19/08 (CAK) Modify subroutine to remove the code that is using
**                   shallow, intermediate, deep, and very deep soil depths to
**                   calculate a weighted average value for calculating
**                   transpiration.  Replace this code with a routine that
**                   will return the soil water potential of the wettest soil
**                   layer within the plant rooting zone to be used for
**                   calculating transpiration.
**    07/25/11 (CAK) Return a 0.0 to 1.0 multiplier instead of the soil water
**                   potential of the wettest soil layer within the plant
**                   rooting zone.
** 
**  INPUTS:
**    flags            - structure containing debugging flags
**    flags->debug     - flag to set debugging mode, 0 = off, 1 = on
**    layers           - soil water soil layer structure
**    layers->fieldc[] - volumetric water content at field capacity for
**                       layer (cm H2O/cm of soil)
**    layers->lbnd[]   - the index of the lower soil water model layer which
**                       corresponds clyr in Century
**    layers->swc[]    - soil water content by layer (cm H2O)
**    layers->width[]  - the thickness of soil water model layers (cm)
**    layers->wiltpt[] - volumetric soil water content at wilting point for
**                       a layer (cm H2O/cm of soil)
**    nlaypg           - number of layers in plant rooting zone
**
**  GLOBAL VARIABLES:
**
**  LOCAL VARIABLES:
**    callname - call name for subroutine
**    ilyr     - current layer in the soil profile
**    rwcf     - relative water content by layer 
**    swp      - 0.0 to 1.0 multiplier
** 
**  OUTPUTS:
**    swp - 0.0 to 1.0 multiplier
**
**  CALLED BY:
**    watrflow()
**
**  CALLS:
**    None
**
*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include "soilwater.h"

    float trwtavg(int *nlaypg, LAYERPAR_SPT layers, FLAG_SPT flags)
    {
      int ilyr;
      float swp;
      double rwcf;
      static char *callname = "trwtavg";

      if (flags->debug > 2) {
        printf("Entering function trwtavg\n");
      }

      /* Find the wettest layer in the plant rooting zone */
      swp = 0.0f;
      for(ilyr=0; ilyr <= layers->lbnd[*nlaypg-1]; ilyr++) {
        rwcf = (layers->swc[ilyr]/layers->width[ilyr] - layers->wiltpt[ilyr])/
               (layers->fieldc[ilyr] - layers->wiltpt[ilyr]);
        swp = (float)max(1.0 - exp(-3.0f * rwcf), swp);
      }

      if (flags->debug > 2) {
        printf("Exiting function trwtavg\n");
      }

      return(swp);
    }
